package com.jiuqi.util.type;

package .tmp;

import com.jiuqi.util.hardware.HardwareUtil;
import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.Arrays;
import java.util.Date;
import java.util.Random;

public final class GUID implements Cloneable, Comparable {
  private static final char[] HEX_CHARS = new char[] { 
      '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 
      'A', 'B', 'C', 'D', 'E', 'F' };
  
  private static final byte[] hardCode;
  
  static {
    Date today = new Date();
    byte[] buf = HardwareUtil.getMachineSign("GUID");
    if (buf == null) {
      BigInteger value = BigInteger.valueOf(today.getTime() ^ System.currentTimeMillis());
      byte[] data = value.toByteArray();
      buf = new byte[8];
      System.arraycopy(data, 0, buf, 0, (data.length > 8) ? 8 : data.length);
    } 
    hardCode = buf;
  }
  
  private static final Random seedProvider = new SecureRandom();
  
  public static final String EMPTY = "00000000000000000000000000000000";
  
  private Random random;
  
  private byte[] code;
  
  private char[] buffer;
  
  private int hashcode;
  
  static {
    seedProvider.setSeed(today.getTime());
  }
  
  public GUID() {
    synchronized (seedProvider) {
      this.random = new Random(seedProvider.nextLong());
    } 
    this.code = new byte[16];
    this.buffer = new char[32];
    this.hashcode = 0;
  }
  
  public void generate() {
    BigInteger v1 = BigInteger.valueOf(System.currentTimeMillis());
    byte[] data = v1.toByteArray();
    if (data.length >= 8) {
      System.arraycopy(data, 0, this.code, 0, 8);
    } else {
      System.arraycopy(data, 0, this.code, 0, data.length);
      System.arraycopy(hardCode, 0, this.code, data.length, 8 - data.length);
    } 
    BigInteger v2 = new BigInteger(64, this.random);
    data = v2.toByteArray();
    System.arraycopy(data, 0, this.code, 16 - data.length, data.length);
    for (int i = 0; i < 8 - data.length; i++)
      this.code[8 + i] = hardCode[8 - i - 1]; 
    this.hashcode = 0;
  }
  
  public byte[] toBytes() {
    byte[] data = new byte[this.code.length];
    System.arraycopy(this.code, 0, data, 0, this.code.length);
    return data;
  }
  
  public String toString() {
    for (int i = 0; i < this.code.length; i++) {
      this.buffer[i * 2] = HEX_CHARS[this.code[i] >> 4 & 0xF];
      this.buffer[i * 2 + 1] = HEX_CHARS[this.code[i] & 0xF];
    } 
    return new String(this.buffer);
  }
  
  public int hashCode() {
    int h = this.hashcode;
    if (h == 0) {
      for (int i = 0; i < this.code.length; i++)
        h = h * 31 + this.code[i]; 
      this.hashcode = h;
    } 
    return h;
  }
  
  public boolean equals(Object obj) {
    if (this == obj)
      return true; 
    if (this == null || obj == null)
      return false; 
    if (!(obj instanceof com.jiuqi.util.type.GUID))
      return false; 
    com.jiuqi.util.type.GUID other = (com.jiuqi.util.type.GUID)obj;
    if (hashCode() != other.hashCode())
      return false; 
    return Arrays.equals(this.code, other.code);
  }
  
  public Object clone() {
    try {
      com.jiuqi.util.type.GUID result = (com.jiuqi.util.type.GUID)super.clone();
      result.code = new byte[16];
      System.arraycopy(this.code, 0, result.code, 0, 16);
      result.buffer = new char[32];
      System.arraycopy(this.buffer, 0, result.buffer, 0, 32);
      return result;
    } catch (CloneNotSupportedException cloneNotSupportedException) {
      return null;
    } 
  }
  
  public int compareTo(Object arg0) {
    byte[] data = ((com.jiuqi.util.type.GUID)arg0).code;
    for (int i = 0; i < 16; i++) {
      int ret = this.code[i] - data[i];
      if (ret != 0)
        return ret; 
    } 
    return 0;
  }
  
  public static String newGUID() {
    com.jiuqi.util.type.GUID guid = new com.jiuqi.util.type.GUID();
    guid.generate();
    return guid.toString();
  }
}
